import typer


class TyperPrint:  # pragma: no cover
    @staticmethod
    def info(msg: str, with_newline=True) -> None:
        typer.secho(msg, fg=typer.colors.BLUE, nl=with_newline)

    @staticmethod
    def success(msg: str, with_newline=True) -> None:
        typer.secho(msg, fg=typer.colors.GREEN, nl=with_newline, bold=True)

    @staticmethod
    def error(msg: str, with_newline=True) -> None:
        typer.secho(msg, fg=typer.colors.RED, nl=with_newline, bold=True)

    @staticmethod
    def warning(msg: str, with_newline=True) -> None:
        typer.secho(msg, fg=typer.colors.YELLOW, nl=with_newline)



from abc import ABC, abstractmethod

import typer
from typer import Typer

from griff.infra.registry.meta_registry import MetaCliRouterRegistry


class CliRouter(ABC, metaclass=MetaCliRouterRegistry):
    def __init__(self):
        self._app = typer.Typer()

    def get_app(self) -> Typer:
        return self._app

    @abstractmethod
    def get_command_group_name(self) -> str:
        raise NotImplementedError



import subprocess

import typer
from injector import inject

from griff.infra.cli.controllers.abstract_db_cli_controller import (
    AbstractDbCliController,
)
from griff.infra.cli.register_cli_command import register_cli_command
from griff.infra.cli.typer_print import TyperPrint
from griff.services.path.path_service import PathService
from griff.settings.griff_settings import GriffSettings
from griff.utils.async_utils import AsyncUtils
from griff.utils.db_tpl_utils import DbTplUtils


class DbTplCliController(AbstractDbCliController):
    @inject
    def __init__(self, settings: GriffSettings):
        super().__init__(settings)

    def get_command_name(self):
        return "db_tpl"

    @register_cli_command(name="generate")
    def generate(self, context: str, noinput: bool = False):
        db_tpl = DbTplUtils(context, settings=self._settings)
        if AsyncUtils.async_to_sync(db_tpl.is_db_exists):
            if noinput is False:
                confirmed = typer.confirm(
                    f"It will regenerate current '{context}' template, are you sure?"
                )
                if not confirmed:
                    TyperPrint.warning("Ok, no pb, see you later !!")
                    raise typer.Exit()
            TyperPrint.info(f"Deleting Db template for context '{context}'")
            self._delete_tpl(db_tpl)
        TyperPrint.info(f"Creating Db template for context '{context}'")
        self._check_context_exists(db_tpl)
        TyperPrint.info("  creating db ...", with_newline=False)
        try:
            AsyncUtils.async_to_sync(db_tpl.create)
            TyperPrint.success(" OK")
        except Exception as e:
            TyperPrint.error(f"  {e}")
            raise typer.Exit()
        TyperPrint.info("  applying migration ...", with_newline=False)
        db_tpl.do_migration()
        TyperPrint.success(" OK")
        self._dump(db_tpl)
        TyperPrint.info("\nTemplate généré !")
        TyperPrint.warning(
            "Pensez à le (re)charger via la commande : python manage.py init-tpl"
        )

    @register_cli_command(name="dump")
    def dump(self, context: str):
        TyperPrint.info(f"Dump Db template for context '{context}'")
        db_tpl = DbTplUtils(context, settings=self._settings)
        self._dump(db_tpl)
        TyperPrint.success("DONE")

    @register_cli_command(name="restore")
    def restore(self, context: str):
        TyperPrint.info(f"Restoring Db template for context '{context}'")
        db_tpl = DbTplUtils(context, settings=self._settings)
        AsyncUtils.async_to_sync(self._restore_db_tpl, db_tpl)
        TyperPrint.success("DONE")

    @register_cli_command(name="delete")
    def delete(self, context: str):
        db_tpl = DbTplUtils(context, settings=self._settings)
        TyperPrint.info(f"Delete Db template for context '{context}'")
        self._check_context_exists(db_tpl)
        self._delete_tpl(db_tpl)
        TyperPrint.success("DONE")

    @register_cli_command(name="migrate")
    def migrate(self, context: str = "all"):
        if context == "all":
            TyperPrint.info("Migrating All Db template")
            contexts = self._settings.bounded_contexts
        else:
            TyperPrint.info(f"Migrating Db template for context '{context}'")
            contexts = [context]

        for context in contexts:
            db_tpl = DbTplUtils(context, settings=self._settings)
            if db_tpl.dump_filename.exists():
                TyperPrint.info(
                    f"  applying migration for context '{context}'...",
                    with_newline=False,
                )
                db_tpl.do_migration()
                TyperPrint.success(" OK")
                self._dump(db_tpl)
            else:
                TyperPrint.warning(f"  no Db template found for context '{context}'")
        TyperPrint.success("DONE")

    def _check_context_exists(self, db_tpl: DbTplUtils):
        if db_tpl.is_context_exists():
            return None
        TyperPrint.error(f"context '{db_tpl.context}' not found")
        raise typer.Exit()

    def _dump(self, db_tpl: DbTplUtils):
        TyperPrint.info(
            f"  dumping db tpl {db_tpl.tpl_db_name} ...", with_newline=False
        )
        PathService().create_missing(db_tpl.dump_filename)
        p = subprocess.run(
            f"PGPASSWORD={self._settings.db.password} "
            f"pg_dump -Fc -Z 9 -U {self._settings.db.user} -h {self._settings.db.host} "
            f"-p {self._settings.db.port} "
            f"--file={db_tpl.dump_filename} {db_tpl.tpl_db_name}",
            shell=True,
        )
        self._check_process_run(p, "dump")
        db_tpl.dump_sql_filename.unlink(missing_ok=True)
        p = subprocess.run(
            f"PGPASSWORD={self._settings.db.password} "
            f"pg_dump --column-inserts "
            f"-U {self._settings.db.user} "
            f"-h {self._settings.db.host} "
            f"-p {self._settings.db.port} "
            f"--file={db_tpl.dump_sql_filename} {db_tpl.tpl_db_name}",
            shell=True,
        )
        self._check_process_run(p, "dump")
        TyperPrint.success(" OK")

    def _delete_tpl(self, db_tpl: DbTplUtils):
        TyperPrint.info("  deleting db ...", with_newline=False)
        try:
            AsyncUtils.async_to_sync(db_tpl.delete)
            TyperPrint.success(" OK")
        except Exception as e:
            TyperPrint.error(f"  {e}")
            raise typer.Exit()
        TyperPrint.info("  deleting dump ...", with_newline=False)
        db_tpl.dump_filename.unlink(missing_ok=True)
        db_tpl.dump_sql_filename.unlink(missing_ok=True)
        TyperPrint.success(" OK")



from abc import ABC, abstractmethod
from typing import Type, List

from griff.context_entry_point import ContextEntryPoint
from griff.domain.common_types import Entity
from griff.infra.repository.repository import Repository
from griff.test_utils.mixins.runtime_test_mixin import RuntimeTestMixin
from griff.test_utils.pytest_runtime_factory import PytestRuntimeFactory
from griff.test_utils.testcases.testcase import TestCase
from griff.utils.exceptions import EntityNotFoundException


class RepositoryTestCase(RuntimeTestMixin, TestCase, ABC):
    @classmethod
    @abstractmethod
    def entry_point_class(cls) -> Type[ContextEntryPoint]:
        pass

    @classmethod
    def runtime_factory(cls) -> PytestRuntimeFactory:
        return super().runtime_factory().repository_test(cls.entry_point_class())

    @staticmethod
    async def run_crud(
        repository: Repository,
        aggregates: List[Entity],
        aggregate_to_update: Entity,
        update_data: dict,
        aggregate_to_delete: Entity,
    ):
        # insert, read
        for aggregate in aggregates:
            await repository.save(aggregate)
        inserted_data = await repository.list_all()
        # update, read
        updated_aggregate = aggregate_to_update.model_copy(update=update_data)
        await repository.save(updated_aggregate)
        after_update = await repository.get_by_id(updated_aggregate.entity_id)
        # delete, read
        assert await repository.get_by_id(aggregate_to_delete.entity_id)
        await repository.delete(aggregate_to_delete)
        try:
            await repository.get_by_id(aggregate_to_delete.entity_id)
            assert False, "aggregate was not deleted"
        except EntityNotFoundException:
            assert True
        return {
            "inserted_data": inserted_data,
            "after_update": after_update,
        }



from griff.appli.command.command import Command
from griff.appli.command.command_handler import CommandResponse
from griff.appli.message.message_dispatcher import MessageDispatcher


class CommandDispatcher(MessageDispatcher[Command, CommandResponse]):
    ...



import pytest_asyncio

from griff.utils.async_utils import AsyncUtils


class AsyncTestMixin(AsyncUtils):
    @pytest_asyncio.fixture(scope="class", autouse=True)
    async def _async_setup_teardown_class(self):
        await self.async_setup_class()
        yield
        await self.async_teardown_class()

    async def async_setup_class(self):
        # ATTENTION le self reçu sera différent de celui async_setup et donc du test
        # => tout attribut affecté ici ne le sera pas dans le context du test
        pass

    async def async_teardown_class(self):
        pass

    @pytest_asyncio.fixture(autouse=True)
    async def _async_setup_teardown_method(self):
        await self.async_setup()
        yield
        await self.async_teardown()

    async def async_setup(self):
        pass

    async def async_teardown(self):
        pass



from typing import Type

from fastapi import FastAPI
from injector import Injector
from loguru import logger

from griff.context_entry_point import ContextEntryPoint
from griff.runtime.components.abstract_runtime_component import (
    RuntimeComponent,
    Runnable,
)


class ContextApiRuntimeComponent(Runnable, RuntimeComponent):
    def __init__(self, context_entry_point_class: Type[ContextEntryPoint]):
        self._entry_point_class = context_entry_point_class

    def initialize(self, injector: Injector):
        entry_point = injector.get(self._entry_point_class)
        context_router = entry_point.get_api_router()
        if context_router is None:
            logger.warning("No API router found for entry point {self._entry_point}")
            return None
        app = injector.get(FastAPI)
        app.include_router(context_router.get_fastapi_router())

    def clean(self, injector: Injector):
        pass

    def start(self, injector: Injector):
        pass

    def stop(self, injector: Injector):
        pass



from types import FunctionType


def register_cli_command(name: str):
    def decorator(function: FunctionType):
        setattr(function, "_endpoint_name", name)
        return function

    return decorator



from loguru import logger

from griff.appli.event.event import Event
from griff.appli.event.event_middleware import EventMiddleware
from griff.appli.message.message_middleware import MessageContext


class EventLoggerMiddleware(EventMiddleware):
    async def dispatch(
        self, message: Event, context: MessageContext | None = None
    ) -> None:
        logger.info(f"dispatch event: {message.short_classname()}")
        logger.debug(message.model_dump())
        await self._next_dispatch(message, context)



import injector

from griff.services.abstract_service import AbstractService
from griff.services.uniqid.generator.uniqid_generator import (
    UniqIdGenerator,
)


class UniqIdService(AbstractService):
    @injector.inject
    def __init__(self, generator: UniqIdGenerator):
        self._generator = generator

    def get(self):
        return self._generator.next_id()

    def reset(self, start_id=1):
        self._generator.reset(start_id)



import copy
from abc import ABC

from injector import inject

from griff.infra.persistence.persistence import Persistence, QueryRowResult
from griff.services.json.json_service import JsonService


class SerializedPersistence(Persistence, ABC):
    @inject
    def __init__(self, json_service: JsonService):
        self.json_service = json_service
        self._metadata_fields = ["created_at", "updated_at"]

    @property
    def _serialized_persistence_fieldname(self):
        return "serialized"

    @property
    def _serialize_excluded_fields(self):
        return []

    def _prepare_to_save(self, data: dict) -> dict:
        data_to_serialize = {
            k: v for k, v in data.items() if not self._is_field_serialize_excluded(k)
        }
        attr_serialized = data_to_serialize.keys()
        json_prepared = self.json_service.to_json_dumpable(data_to_serialize)
        return {
            "entity_id": data["entity_id"],
            self._serialized_persistence_fieldname: self.json_service.dump(
                json_prepared
            ),
            **{k: v for k, v in data.items() if k not in attr_serialized},
        }

    def _prepare_row_result(self, result: QueryRowResult) -> QueryRowResult:
        result_copy = copy.deepcopy(result)
        serialized = result_copy.pop(self._serialized_persistence_fieldname)
        return {**result_copy, **self.json_service.load_from_str(serialized)}

    def _is_field_serialize_excluded(self, field_name) -> bool:
        return (
            field_name in self._metadata_fields
            or field_name in self._serialize_excluded_fields
        )



from injector import inject

from griff.appli.command.command import Command
from griff.appli.command.command_dispatcher import CommandDispatcher
from griff.appli.command.command_handler import CommandResponse, CommandHandler
from griff.appli.message.message_bus import MessageBus


class CommandBus(MessageBus[Command, CommandResponse, CommandHandler]):
    @inject
    def __init__(self, dispatcher: CommandDispatcher) -> None:
        super().__init__(dispatcher)



import typing
from abc import ABC, abstractmethod
from pprint import pprint
from typing import Optional, Union, Type
from urllib.parse import urlencode

import httpx
import orjson
from fastapi import FastAPI
from fastapi import Response, status
from pydantic import BaseModel, Field, field_validator, model_validator

from griff.appli.event.event_handler import FakeEventHandler
from griff.context_entry_point import ContextEntryPoint
from griff.infra.persistence.persistence import QueryRowResults
from griff.infra.repository.repository import Repository
from griff.services.json.json_service import JsonService
from griff.test_utils.mixins.runtime_test_mixin import RuntimeTestMixin
from griff.test_utils.pytest_runtime_factory import PytestRuntimeFactory
from griff.test_utils.testcases.testcase import TestCase


class ApiClientParams(BaseModel):
    data_json: Optional[Union[list, dict]] = Field(None, alias="json")
    data: Optional[Union[list, dict]] = Field(None, alias="data")
    files: Optional[dict] = None
    auto_auth: bool = True
    access_token: Optional[str] = None
    headers: Optional[dict] = None

    @model_validator(mode="after")
    def check_auto_auth(cls, values):
        if values.auto_auth and values.access_token is None:
            raise ValueError("Missing Jwt Access Token for automatic authentication")
        return values

    @field_validator("headers")
    @classmethod
    def set_header(cls, value):
        return value or {}

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if self.auto_auth:
            self.headers["Authorization"] = f"Bearer {self.access_token}"
        self._json_service = JsonService()

    def to_method_kwargs(self) -> dict:
        kwargs = super().dict(
            exclude={"auto_auth", "access_token"}, by_alias=True, exclude_none=True
        )
        if "json" in kwargs:
            kwargs["json"] = self._json_service.to_json_dumpable(kwargs["json"])
        if "data" in kwargs:
            kwargs["data"] = self._json_service.to_json_dumpable(kwargs["data"])
        return kwargs


# noinspection PyMethodOverriding
class FastApiTestClient(httpx.AsyncClient):
    default_access_token: str | None = None

    @typing.no_type_check
    async def post(
        self,
        url,
        json: Optional[Union[list, dict]] = None,
        data: Optional[Union[list, dict]] = None,
        files: Optional[dict] = None,
        auto_auth: bool = True,
        access_token=None,
        headers: dict | None = None,
    ) -> Response:
        params = ApiClientParams(
            json=json,
            data=data,
            files=files,
            auto_auth=auto_auth,
            headers=headers,
            access_token=access_token if access_token else self.default_access_token,
        ).to_method_kwargs()
        return await super().post(url=url, **params)

    @typing.no_type_check
    async def get(
        self,
        url,
        auto_auth: bool = True,
        access_token=None,
        headers: dict | None = None,
    ) -> Response:
        params = ApiClientParams(
            auto_auth=auto_auth,
            headers=headers,
            access_token=access_token if access_token else self.default_access_token,
        ).to_method_kwargs()
        return await super().get(url=url, **params)

    @typing.no_type_check
    async def put(
        self,
        url,
        json: Optional[Union[list, dict]] = None,
        files: Optional[dict] = None,
        auto_auth: bool = True,
        access_token=None,
        headers: dict | None = None,
    ) -> Response:
        params = ApiClientParams(
            json=json,
            files=files,
            auto_auth=auto_auth,
            headers=headers,
            access_token=access_token if access_token else self.default_access_token,
        ).to_method_kwargs()
        return await super().put(url=url, **params)

    @typing.no_type_check
    async def patch(
        self,
        url,
        json: Optional[Union[list, dict]] = None,
        files: Optional[dict] = None,
        auto_auth: bool = True,
        access_token=None,
        headers: dict | None = None,
    ) -> Response:
        params = ApiClientParams(
            json=json,
            files=files,
            auto_auth=auto_auth,
            headers=headers,
            access_token=access_token if access_token else self.default_access_token,
        )
        return await super().patch(url=url, **params.to_method_kwargs())

    @typing.no_type_check
    async def delete(
        self,
        url,
        auto_auth: bool = True,
        access_token=None,
        headers: dict | None = None,
    ) -> Response:
        params = ApiClientParams(
            auto_auth=auto_auth,
            headers=headers,
            access_token=access_token if access_token else self.default_access_token,
        ).to_method_kwargs()
        return await super().delete(url=url, **params)


class ApiTestCase(RuntimeTestMixin, TestCase, ABC):
    app: FastAPI
    client: FastApiTestClient

    @classmethod
    @abstractmethod
    def entry_point_class(cls) -> Type[ContextEntryPoint]:
        pass

    @classmethod
    def runtime_factory(cls) -> PytestRuntimeFactory:
        return super().runtime_factory().api_test(cls.entry_point_class())

    def setup_method(self):
        super().setup_method()
        self.app = self.get_injected(FastAPI)
        self.client = FastApiTestClient(app=self.app, base_url="http://test")

    def assert_response_equals_resultset(
        self, response: Response, remove_paths: Optional[list] = None
    ):
        return self.assert_equals_resultset(
            self.prepare_response_for_resultset(response), remove_paths=remove_paths
        )

    def assert_response_status_code(
        self, response: Response, status=status.HTTP_200_OK
    ):
        assert status == response.status_code, (
            f"expected status {status} got {response.status_code}\n"
            f"response body: {self.get_response_body(response)}"
        )

    @classmethod
    def prepare_response_for_resultset(cls, response: Response):
        try:
            return {
                "status_code": response.status_code,
                "body": cls.get_response_body(response),
            }
        except Exception as e:
            pprint(cls.get_response_body(response))
            raise e

    @typing.no_type_check
    def list_api_routes(self, api_name, except_routes: list | None = None):
        if except_routes is None:
            except_routes = []
        return [
            route
            for route in self.app.routes
            if f"{api_name}:" in route.name and route.name not in except_routes
        ]

    def reverse_url(self, name, query_kwargs=None, **kwargs):
        """
        Url reverse from FastAPI routes

        Usage:
            reverse(
                <url_name>,
                pk=123,
                query_kwargs={'key':'value', 'k2': 'v2'}
            )
            => url/123?key=value&k2=v2

        Args:
            name: route name
            query_kwargs: optional query params
            **kwargs: route url params

        Returns:
            str: relative url built from router name and params with query params
            if asked
        """
        base_url = self.app.url_path_for(name, **kwargs)
        if query_kwargs:
            return f"{base_url}?{urlencode(query_kwargs, doseq=True)}"

        return base_url

    @classmethod
    def get_response_body(cls, response: Response):
        content = response.content  # type: ignore
        return orjson.loads(content) if content else None

    async def prepare_success_resultset(
        self,
        response: Response,
        repository: Repository,
        event_handler: FakeEventHandler,
    ):
        results: QueryRowResults = await repository._persistence.list_all()
        return {
            "response": self.prepare_response_for_resultset(response),
            "persistence": results,
            "handled_events": event_handler.list_events_handled(),
        }



from fastapi import Request, status
from fastapi.exceptions import RequestValidationError
from pydantic_i18n import PydanticI18n
from starlette.responses import JSONResponse

from griff.infra.fastapi.exception_handlers.validation_exception_simplifier import (
    simplify_errors,
)
from griff.utils.errors import (
    get_error_by_status_code,
)


class PydanticValidationExceptionHandler:
    def __init__(self, pydantic_i18n: PydanticI18n) -> None:
        self._i18n = pydantic_i18n

    async def dispatch(
        self, request: Request, exc: RequestValidationError
    ) -> JSONResponse:
        current_locale = request.query_params.get("locale", self._i18n.default_locale)
        errors = self._i18n.translate(list(exc.errors()), current_locale)
        error_class = get_error_by_status_code(status.HTTP_422_UNPROCESSABLE_ENTITY)
        error = error_class(details=simplify_errors(errors))
        return JSONResponse(status_code=error.code, content=error.model_dump())



import abc


class UniqIdGenerator(abc.ABC):
    @abc.abstractmethod
    def next_id(self) -> str:
        ...  # pragma: no cover

    @abc.abstractmethod
    def reset(self, start_id: int = 1) -> None:  # pragma: no cover
        pass



from typing import List, TypeVar, Generic

from fastapi import FastAPI
from injector import Injector
from typer import Typer

from griff.runtime.components.abstract_runtime_component import (
    RuntimeComponent,
    InjectBindable,
    Runnable,
    AsyncRunnable,
)
from griff.settings.griff_settings import GriffSettings

S = TypeVar("S", bound=GriffSettings)


class Runtime(Generic[S]):
    def __init__(
        self,
        settings: S,
        components: List[RuntimeComponent],
    ) -> None:
        self._settings = settings
        self._components = components
        self._injector = Injector(self._inject_bindable_components)

    def initialize(self):
        for component in self._runnable_components:
            component.initialize(self._injector)

    def clean(self):
        for component in self._runnable_components:
            component.clean(self._injector)

    def start(self):
        for component in self._runnable_components:
            component.start(self._injector)

    def stop(self):
        for component in self._runnable_components:
            component.stop(self._injector)

    async def async_start(self):
        for component in self._async_runnable_components:
            await component.async_start(self._injector)

    async def async_stop(self):
        for component in self._async_runnable_components:
            await component.async_stop(self._injector)

    def get_injector(self) -> Injector:
        return self._injector

    def get_settings(self) -> S:
        return self._settings

    def get_cli(self) -> Typer:  # pragma: no cover
        return self._injector.get(Typer)

    def get_fastapi_app(self) -> FastAPI:  # pragma: no cover
        app = self._injector.get(FastAPI)
        app.add_event_handler("startup", self.async_start)
        app.add_event_handler("shutdown", self.async_stop)
        return app

    @property
    def _runnable_components(self):
        return [c for c in self._components if isinstance(c, Runnable)]

    @property
    def _async_runnable_components(self):
        return [c for c in self._components if isinstance(c, AsyncRunnable)]

    @property
    def _inject_bindable_components(self):
        return [c.configure for c in self._components if isinstance(c, InjectBindable)]



from abc import ABC, abstractmethod
from typing import Callable, List

from fastapi import APIRouter

from griff.infra.api.api_controller import ApiController
from griff.infra.registry.meta_registry import MetaApiRouterRegistry


class ApiRouter(ABC, metaclass=MetaApiRouterRegistry):
    def __init__(self):
        self._router = APIRouter(prefix=self._route_prefix, tags=self._route_tags)
        for ctrl in self._list_controllers():
            for endpoint in ctrl.get_endpoints():
                self.add_endpoint(**endpoint)

    def get_fastapi_router(self) -> APIRouter:
        return self._router

    def add_endpoint(self, route: str, method: str, func: Callable, return_code: int):
        self._router.add_api_route(
            path=route, endpoint=func, methods=[method], status_code=return_code
        )

    @abstractmethod
    def _list_controllers(self) -> List[ApiController]:  # pragma: no cover
        ...

    @property
    @abstractmethod
    def _route_prefix(self) -> str:  # pragma: no cover
        pass

    @property
    @abstractmethod
    def _route_tags(self) -> List[str]:  # pragma: no cover
        pass



from griff.services.uniqid.generator.fake_uniqid_generator import FakeUniqIdGenerator
from griff.services.uniqid.uniqid_service import UniqIdService
from griff.test_utils.testcases.testcase import TestCase


class AggregateTestCase(TestCase):
    uniqid_service: UniqIdService

    @classmethod
    def setup_class(cls):
        super().setup_class()
        cls.uniqid_service = UniqIdService(
            FakeUniqIdGenerator(999999999999999999999999999999990001)
        )

    def setup_method(self):
        super().setup_method()
        self.uniqid_service.reset()



from abc import ABC, abstractmethod

from griff.appli.event.event import Event
from griff.appli.message.message_middleware import MessageMiddleware, MessageContext


class EventMiddleware(MessageMiddleware[Event, None], ABC):
    @abstractmethod
    async def dispatch(
        self, message: Event, context: MessageContext | None = None
    ) -> None:
        pass



from ulid import ULID

from griff.services.uniqid.generator.uniqid_generator import (
    UniqIdGenerator,
)


class UlidUniqIdGenerator(UniqIdGenerator):
    def next_id(self) -> str:
        return str(ULID())

    def reset(self, start_id: int = 1) -> None:  # pragma: no cover
        pass



from injector import inject

from griff.appli.command.command import Command
from griff.appli.command.command_handler import CommandResponse
from griff.appli.command.command_middleware import CommandMiddleware
from griff.appli.message.message_middleware import MessageContext
from griff.services.db.db_service import DbService


class CommandUowMiddleware(CommandMiddleware):
    @inject
    def __init__(self, db_service: DbService):
        super().__init__()
        self._db_service = db_service

    async def dispatch(
        self, message: Command, context: MessageContext | None = None
    ) -> CommandResponse:
        async with self._db_service.transaction():
            response = await self._next_dispatch(message, context)
        return response



from abc import ABC, abstractmethod
from typing import Self

from griff.runtime.factories.abstract_runtime_factory import AbstractRuntimeFactory


class AppRuntimeFactory(AbstractRuntimeFactory, ABC):
    def web_runtime(self) -> Self:
        (
            self.with_log_level()
            .with_default_services()
            .with_service_locator()
            .with_db()
            .with_command_bus()
            .with_query_bus()
            .with_event_bus()
            .with_app_event_bus()
            .with_fastapi()
            .with_all_apis()
        )
        return self

    def cli_runtime(self) -> Self:
        (
            self.with_log_level()
            .with_default_services()
            .with_service_locator()
            .with_db()
            .with_command_bus(for_cli=True)
            .with_query_bus(for_cli=True)
            .with_event_bus(for_cli=True)
            .with_app_event_bus(for_cli=True)
            .with_common_cli()
            .with_all_clis()
        )
        return self

    @abstractmethod
    def with_all_apis(self) -> Self:
        ...

    @abstractmethod
    def with_all_clis(self) -> Self:
        ...



from abc import ABC, abstractmethod
from typing import TypeVar, Generic, Type

from injector import inject
from pydantic import ValidationError

from griff.domain.common_types import Entity
from griff.infra.persistence.dict_persistence import DictPersistence
from griff.infra.persistence.persistence import (
    Persistence,
    QueryRowResult,
)
from griff.services.date.date_service import DateService
from griff.utils.exceptions import EntityNotFoundException

A = TypeVar("A", bound=Entity)


class FakeRepositoryMixin:  # pragma: no cover
    _persistence: DictPersistence

    def reset(self):
        self._persistence.reset()


class Repository(Generic[A], ABC):
    @inject
    @abstractmethod
    def __init__(
        self,
        persistence: Persistence,
        date_service: DateService,
    ):
        self._date_service = date_service
        self._persistence = persistence

    async def get_by_id(self, entity_id: str) -> A | None:
        raw_data = await self._get_by_id(entity_id)
        if raw_data is not None:
            return self._hydrate_result(raw_data)

        raise EntityNotFoundException(
            f"Entity '{self._aggregate_class(raw_data)._entity_label()}' "
            f"(entity_id={entity_id}) not found"
        )

    async def save(self, aggregate: A) -> None:
        existing_entity = await self._get_by_id(aggregate.entity_id)
        prepared_data = aggregate.model_dump()
        if existing_entity is None:
            prepared_data = self._create_metadata(prepared_data)
            await self._persistence.insert(prepared_data)
            return None
        prepared_data = self._update_metadata(prepared_data)
        await self._persistence.update(prepared_data)
        return None

    async def delete(self, aggregate: A) -> None:
        await self.get_by_id(aggregate.entity_id)
        await self._persistence.delete(aggregate.entity_id)

    async def list_all(self) -> list[A]:
        raw_data = await self._persistence.list_all()
        return [self._hydrate_result(row) for row in raw_data if row]

    async def run_query(self, query_name: str, **query_params):
        raw_data = await self._persistence.run_query(query_name, **query_params)
        if raw_data is None:
            return None
        if isinstance(raw_data, list):
            return [self._hydrate_result(row) for row in raw_data if row]
        return self._hydrate_result(raw_data)

    async def _get_by_id(self, entity_id: str) -> dict | None:
        try:
            return await self._persistence.get_by_id(entity_id)
        except ValueError:
            return None

    def reset(self):  # pragma: no cover
        # Only for testing purpose
        pass

    def _hydrate_aggregate(self, raw_data: dict) -> A:
        aggregate_class = self._aggregate_class(raw_data)
        try:
            aggregate = aggregate_class.model_validate(raw_data)
        except ValidationError as e:
            raise RuntimeError(
                f"Data from persistence are not valid, hydratation "
                f"{aggregate_class._entity_label()} impossible : {e.errors()}"
            )

        return aggregate

    def _hydrate_result(self, result: QueryRowResult) -> A:
        return self._hydrate_aggregate(result)

    def _create_metadata(self, target: dict) -> dict:
        now = self._date_service.now().to_datetime()
        target["created_at"] = now
        target["updated_at"] = now
        return target

    def _update_metadata(self, target: dict) -> dict:
        # to be sure
        target.pop("created_at", None)
        target["updated_at"] = self._date_service.now().to_datetime()
        return target

    @staticmethod
    @abstractmethod
    def _aggregate_class(raw_data: dict | None = None) -> Type[A]:  # pragma: no cover
        ...



import jwt
from injector import inject, singleton
from jwt import DecodeError

from griff.services.abstract_service import AbstractService
from griff.services.date.date_service import DateService
from griff.services.jwt.jwt_models import DecodedJwtToken, JwtTokens
from griff.services.jwt.jwt_settings import JwtSettings
from griff.utils.exceptions import UnauthorizedException


class InvalidTokenException(UnauthorizedException):
    default_message = "Invalid token, impossible to check"


@singleton
class JwtService(AbstractService):
    @inject
    def __init__(self, settings: JwtSettings, date_service: DateService):
        self._settings = settings
        self._algorithm = self._settings.algorithm.value
        self._date_service = date_service

    def create_jwt_tokens(self, payload: dict, ts: float | None = None) -> JwtTokens:
        if ts is None:
            ts = self._date_service.now().to_timestamp()

        access_token = self._create_access_token(payload, ts)
        refresh_token = self._create_refresh_token(payload, ts)

        return JwtTokens(access_token=access_token, refresh_token=refresh_token)

    def decode_access_token(self, access_token: str) -> DecodedJwtToken:
        return self._decode_token(access_token, self._settings.access_secret_key)

    def decode_refresh_token(self, refresh_token: str) -> DecodedJwtToken:
        return self._decode_token(refresh_token, self._settings.refresh_secret_key)

    def _create_access_token(self, payload: dict, ts: float) -> str:
        prepared_payload = self._prepare_payload(
            payload, ts, self._settings.access_token_lifetime_seconds
        )
        return jwt.encode(
            prepared_payload,
            self._settings.access_secret_key,
            algorithm=self._algorithm,
        )

    def _create_refresh_token(self, payload: dict, ts: float) -> str:
        prepared_payload = self._prepare_payload(
            payload, ts, self._settings.refresh_token_lifetime_seconds
        )
        return jwt.encode(
            prepared_payload,
            self._settings.refresh_secret_key,
            algorithm=self._algorithm,
        )

    def _decode_token(self, token: str, secret_key: str) -> DecodedJwtToken:
        try:
            decoded_token = jwt.decode(token, secret_key, algorithms=[self._algorithm])
        except DecodeError:
            raise InvalidTokenException()

        token_expires_ts = decoded_token.pop("expires", 0)
        now_ts = self._date_service.now().to_timestamp()
        has_expired = token_expires_ts < now_ts
        return DecodedJwtToken(payload=decoded_token, has_expired=has_expired)

    @staticmethod
    def _prepare_payload(payload: dict, ts: float, token_lifetime: int):
        return {**payload, "expires": ts + token_lifetime}



import inspect
from typing import List, Callable, Any


def get_decorators(function: Callable[[Any], Any]) -> List[str]:
    """Returns list of decorators names

    Args:
        function (Callable): decorated method/function

    Return:
        List of decorators as strings

    Example:
        Given:

        @my_decorator
        @another_decorator
        def decorated_function():
            pass

        >> get_decorators(decorated_function)
        ['@my_decorator', '@another_decorator']

    """
    source = inspect.getsource(function)
    if "@" not in source:
        return []
    source = source.replace("\n", "")
    index = source.find("def ")
    async_index = source.find("async ")
    if index > async_index != -1:
        index = async_index
    return [
        f"@{line.replace(' ', '')}"
        for line in source[:index].strip().split("@")
        if line != ""
    ]


def find_bound_method_to_object(obj: object, method_name: str) -> Any:
    members = inspect.getmembers(obj)
    return [mem for mem in members if mem[0] == method_name][0][1]



from functools import lru_cache
from pathlib import Path

from pydantic import Field

from griff.settings.griff_settings import GriffSettings


@lru_cache
def get_root_dir():
    return str(Path(__file__).resolve().parent.parent)


class PytestSettings(GriffSettings):
    project_dir: str = Field(default_factory=get_root_dir)



from typing import Type

from injector import Injector, Binder, singleton

from griff.appli.message.message_bus import ListMiddlewares
from griff.appli.query.middlewares.query_logger_middleware import QueryLoggerMiddleware
from griff.appli.query.query_bus import QueryBus
from griff.appli.query.query_middleware import QueryMiddleware
from griff.infra.registry.meta_registry import (
    MetaQueryHandlerRegistry,
)
from griff.runtime.components.abstract_runtime_component import (
    RuntimeComponent,
    Runnable,
    InjectBindable,
)


class QueryBusRuntimeComponent(Runnable, InjectBindable, RuntimeComponent):
    def __init__(self, middlewares: list[Type[QueryMiddleware]]):
        self._middlewares = middlewares

    def configure(self, binder: Binder) -> None:
        binder.bind(QueryBus, to=QueryBus, scope=singleton)

    def initialize(self, injector: Injector):
        query_bus = injector.get(QueryBus)
        middlewares: ListMiddlewares = [injector.get(m) for m in self._middlewares]
        handlers = [injector.get(h) for h in MetaQueryHandlerRegistry.list_types()]
        query_bus.initialize(handlers, middlewares)

    def clean(self, injector: Injector):
        pass

    def start(self, injector: Injector):
        pass

    def stop(self, injector: Injector):
        pass

    @staticmethod
    def _list_middlewares() -> list[Type[QueryMiddleware]]:
        return [QueryLoggerMiddleware]



import hashlib
import json
from typing import Any

from griff.services.json.json_service import JsonService


class StubMixin:
    def __init__(self):
        self._calls = {}
        self._stub = {}
        self._stub_with_arguments = {}

    def reset_stub(self):
        self._calls = {}
        self._stub = {}

    def list_calls(self):
        return self._calls

    def call_count(self, method) -> int:
        return len([c for c in self._calls if method in c])

    def stub(
        self, method, response: Any | None = None, responses: list | None = None
    ):  # pragma: no cover
        if responses is None:
            self._stub[method] = [response]
            return None
        self._stub[method] = responses

    def _hash_payload(self, payload: dict):
        dhash = hashlib.md5()
        encoded = json.dumps(
            JsonService().to_json_dumpable(payload), sort_keys=True
        ).encode()
        dhash.update(encoded)
        return dhash.hexdigest()

    def stub_with_argument(self, method, arguments: dict, response):  # pragma: no cover
        _hashed_arguments = self._hash_payload(arguments)
        if method not in self._stub_with_arguments:
            self._stub_with_arguments[method] = {}
        self._stub_with_arguments[method][_hashed_arguments] = response

    def _call_stub(self, default_response=None) -> Any:
        name, payload = self._get_caller()

        self._register_call(name, payload)
        if name in self._stub:
            return self._render_stub(name)

        _hashed_arguments = self._hash_payload(payload)
        if (
            name in self._stub_with_arguments
            and _hashed_arguments in self._stub_with_arguments[name]
        ):
            return self._render_stub_with_argument(name, _hashed_arguments)

        return default_response

    def _register_call(self, method, payload):
        if method not in self._calls:
            self._calls[method] = []
        self._calls[method].append(payload)

    @staticmethod
    def _get_caller():
        import inspect

        curframe = inspect.currentframe()
        calframe = inspect.getouterframes(curframe, 2)
        method_name = calframe[2][3]
        payload = calframe[2].frame.f_locals
        if "self" in payload:
            payload.pop("self")
        return method_name, payload

    def _render_stub_with_argument(self, method, hashed_arguments):  # pragma: no cover
        response = self._stub_with_arguments[method].pop(hashed_arguments)
        if isinstance(response, Exception):
            raise response
        return response

    def _render_stub(self, method):  # pragma: no cover
        response = self._stub[method].pop(0)
        if not self._stub[method]:
            self._stub.pop(method)
        if isinstance(response, Exception):
            raise response
        return response



from abc import ABC, abstractmethod
from typing import Type, TypeVar, Generic

from griff.appli.event.event_handler import EventHandler
from griff.infra.persistence.persistence import Persistence
from griff.test_utils.mixins.runtime_test_mixin import RuntimeTestMixin
from griff.test_utils.pytest_runtime_factory import PytestRuntimeFactory
from griff.test_utils.testcases.testcase import TestCase

CH = TypeVar("CH", bound=EventHandler)


class AppEventHandlerTestCase(Generic[CH], RuntimeTestMixin, TestCase, ABC):
    handler: CH

    @classmethod
    def runtime_factory(cls) -> PytestRuntimeFactory:
        return super().runtime_factory().event_test_handler()

    def setup_method(self):
        super().setup_method()
        self.handler = self.get_injected(self.handler_class)

    @staticmethod
    async def prepare_success_resultset(persistence: Persistence):
        return {"persistence": await persistence.list_all()}

    @property
    @abstractmethod
    def handler_class(self) -> Type[CH]:
        pass



from pathlib import Path
from typing import List

import aiosql
from injector import inject
from loguru import logger

from griff.services.abstract_service import AbstractService
from griff.services.db.db_service import DbService
from griff.services.query_runner.query_runner_settings import QueryRunnerSettings


class QueryRunnerService(AbstractService):
    @inject
    def __init__(self, db_service: DbService, settings: QueryRunnerSettings):
        self._db_service = db_service
        self._sql_queries = None
        self._settings = settings

    async def run_query(self, query_name, **query_params):
        query = self.check_query_exists(query_name)
        async with self._db_service.connection() as conn:
            try:
                sql = query.sql.replace("\n", " ")
                logger.debug(f"'{query_name}':SQL: {sql}\nparams : {query_params}")
                results = await query(conn=conn, **query_params)
                return self._format_results(results)
            except Exception as exec_exception:
                raise RuntimeError(str(exec_exception))

    async def run_many_query(self, query_name, queries_params: List[dict]):
        query = self.check_query_exists(query_name)
        async with self._db_service.connection() as conn:
            try:
                results = await query(conn, queries_params)
                return self._format_results(results)
            except Exception as exec_exception:
                raise RuntimeError(str(exec_exception))

    def set_sql_queries(self, relative_sql_path: str) -> None:
        sql_path = Path(self._settings.project_dir).joinpath(relative_sql_path)
        self._sql_queries = aiosql.from_path(
            sql_path=str(sql_path), driver_adapter=self._settings.driver
        )

    def check_query_exists(self, query_name: str):
        try:
            return getattr(self._sql_queries, query_name)
        except AttributeError as e:
            raise RuntimeError(f"query '{query_name}' not found: {e}")

    @staticmethod
    def _format_results(results) -> dict | List[dict] | None:
        if results is None or isinstance(results, (str, int, float)):
            return None
        if isinstance(results, list):
            return [dict(row) for row in results]
        return dict(results)



from injector import Binder, Injector

from griff.runtime.components.abstract_runtime_component import (
    RuntimeComponent,
    InjectBindable,
    AsyncRunnable,
)
from griff.services.db.db_service import DbService
from griff.services.db.db_test_service import DbTestService
from griff.utils.async_utils import AsyncUtils


class DbTestInTransactionRuntimeComponent(
    AsyncUtils, InjectBindable, AsyncRunnable, RuntimeComponent
):
    """Seulement à usage des tests unitaires"""

    def configure(self, binder: Binder) -> None:
        binder.bind(DbService, DbTestService)

    async def async_init(self, injector: Injector):
        pass

    async def async_shutdown(self, injector: Injector):
        pass

    async def async_start(self, injector: Injector):
        db_service = injector.get(DbService)
        await db_service.start_transaction()

    async def async_stop(self, injector: Injector):
        db_service = injector.get(DbService)
        await db_service.rollback_transaction()



import threading
from contextlib import asynccontextmanager
from contextvars import ContextVar
from typing import Any

from injector import inject, singleton
from loguru import logger

from griff.services.abstract_service import AbstractService
from griff.services.db.providers.db_provider import (
    DbProvider,
)


@singleton
class DbService(AbstractService):
    @inject
    def __init__(self, provider: DbProvider):
        self._provider = provider
        self._current_connection = ContextVar("current_connection", default=None)
        self._debug = False

    async def start(self):
        self.debug("--->STARTING DB")
        await self._provider.start()

    async def stop(self):
        self.debug("--->CLOSING DB")
        await self._provider.stop()

    @asynccontextmanager
    async def connection(self):
        conn = self._get_current_connection()
        if conn is not None:
            self.debug(f"got TRANSACTION connection {id(conn)}")
            yield conn
        else:
            conn = await self._provider.get_connection()
            try:
                self.debug(f"got connection {id(conn)}")
                yield conn
            except Exception as e:
                raise e
            finally:
                self.debug(f"close connection {id(conn)}")
                await self._provider.close_connection(conn)

    @asynccontextmanager
    async def transaction(self):
        if self._get_current_connection() is not None:
            raise RuntimeError("A transaction is already running")

        conn = await self._provider.get_connection()
        token = self._set_current_connection(conn)
        self.debug(f"TRANSACTION START on {id(conn)}")
        try:
            async with self._provider.transaction(conn):
                yield conn
        finally:
            self.debug(f"release TRANSACTION connection {id(conn)}")
            await self._provider.close_connection(conn)
            self._reset_current_connection(token)

    async def start_transaction(self):
        if self._get_current_connection() is not None:
            raise RuntimeError("A transaction is already running")

        conn = await self._provider.get_connection()
        self.debug(f"TRANSACTION START on {id(conn)}")
        self._set_current_connection(conn)
        await self._provider.start_transaction(conn)

    async def commit_transaction(self):
        conn = self._get_current_connection()

        if conn is None:
            raise RuntimeError("No transaction active")

        await self._provider.commit_transaction(conn)
        self._set_current_connection(None)
        await self._provider.close_connection(conn)

    async def rollback_transaction(self):
        conn = self._get_current_connection()
        if conn is None:
            raise RuntimeError("No transaction active")

        await self._provider.rollback_transaction(conn)
        self._set_current_connection(None)
        await self._provider.close_connection(conn)

    def _get_current_connection(self) -> Any:
        return self._current_connection.get()

    def _set_current_connection(self, connection):
        return self._current_connection.set(connection)

    def _reset_current_connection(self, token):
        self._current_connection.reset(token)

    def debug(self, msg):  # pragma: no cover
        if self._debug is True:
            logger.debug(f" PG : {threading.current_thread().ident} {msg}")



from abc import ABC, abstractmethod
from importlib import import_module
from pathlib import Path
from typing import Protocol, runtime_checkable

from injector import Binder, Injector


def import_handlers(project_dir: str, event_handlers_dir: Path) -> None:
    skip_dirs = ["__pycache__", "_tests"]
    event_handler_files = [
        str(f)
        for f in event_handlers_dir.rglob("*.py")
        if set(f.parts).isdisjoint(skip_dirs) and not f.name.startswith("_")
    ]

    for file in event_handler_files:
        relative_path = file.replace(f"{str(project_dir)}/", "")
        module = relative_path.replace(".py", "").replace("/", ".")
        package = module.split(".")[-1].replace("_", " ").title().replace(" ", "")

        if module in globals() and hasattr(globals()[module], package):
            continue
        imported_module = import_module(module)
        if hasattr(imported_module, package):
            globals().update({package: getattr(imported_module, package)})
            continue
        raise RuntimeError(f"package {package} not found in {module}")
    return None


@runtime_checkable
class Runnable(Protocol):
    @abstractmethod
    def initialize(self, injector: Injector) -> None:
        ...

    @abstractmethod
    def start(self, injector: Injector) -> None:
        ...

    @abstractmethod
    def stop(self, injector: Injector) -> None:
        ...

    @abstractmethod
    def clean(self, injector: Injector) -> None:
        ...


@runtime_checkable
class AsyncRunnable(Protocol):
    @abstractmethod
    async def async_start(self, injector: Injector) -> None:
        ...

    @abstractmethod
    async def async_stop(self, injector: Injector) -> None:
        ...


@runtime_checkable
class InjectBindable(Protocol):
    @abstractmethod
    def configure(self, binder: Binder) -> None:
        ...


class RuntimeComponent(ABC):
    def __str__(self) -> str:
        return self.__class__.__name__



import shutil
from pathlib import Path

from griff.services.abstract_service import AbstractService
from griff.utils import exceptions


class PathService(AbstractService):
    def check_exists(self, path: str | Path) -> Path:
        path_path = self.to_path(path)
        if path_path.exists():
            return path_path
        raise exceptions.NotFoundException(f"'{path}' does not exist")

    def create_missing(self, path: str | Path) -> None:
        """
        create missing directories for directory or path
        """
        path_path = self.to_path(path)
        if path_path.suffix:
            path_path = path_path.parent
        path_path.mkdir(parents=True, exist_ok=True)

    def read_file(self, filename: str | Path) -> str:
        filename_path = self.check_exists(filename)
        with filename_path.open("r") as fd:
            return fd.read()

    def write_file(self, filename: str | Path, content: str) -> None:
        filename_path = self.to_path(filename)
        self.check_exists(filename_path.parent)
        with filename_path.open("w") as f:
            f.write(content)

    @staticmethod
    def copy_file(src: str | Path, dst: str | Path):
        shutil.copy(str(src), str(dst))

    @staticmethod
    def to_path(path) -> Path:
        """Get Path instance if path is not"""
        if isinstance(path, Path):
            return path
        return Path(path)



import datetime
from abc import ABC
from typing import Any

from pydantic import computed_field, Field

from griff.appli.message.message import Message
from griff.services.date.date_service import DateService
from griff.services.service_locator.service_locator import ServiceLocator
from griff.services.uniqid.uniqid_service import UniqIdService


class Event(Message, ABC):
    id: str = Field(default_factory=lambda: ServiceLocator.get(UniqIdService).get())
    payload: Any
    created_at: datetime.datetime = Field(
        default_factory=lambda: ServiceLocator.get(DateService).to_datetime()
    )

    @computed_field
    def event_name(self) -> str:  # pragma: no cover
        return self.short_classname()



import copy
import datetime
from typing import Any, Union

import arrow
from arrow import Arrow

DateTimeIn = Union[str, datetime.datetime, datetime.date, int, float, Arrow]


# todofsc: pouvoir l'utiliser comme type pydantic
class DateTime:
    def __init__(self, datetime: DateTimeIn):
        self._value = (
            datetime if isinstance(datetime, Arrow) else self._check_is_valid(datetime)
        )
        self._timezone = "UTC"

    def to_mysql_date(self) -> str:
        return self._format("YYYY-MM-DD")

    def to_mysql_datetime(self) -> str:
        return self._format("YYYY-MM-DD HH:mm:ss.SSSSSS")

    def to_date(self) -> datetime.date:
        return self._value.date()

    def to_datetime(self) -> datetime.datetime:
        return self._value.to(self._timezone).datetime

    def to_timestamp(self) -> float:
        return self._value.timestamp()

    def copy(self):
        return copy.copy(self)

    def format(self, format: str):
        return self._format(format)

    @classmethod
    def _check_is_valid(cls, value: Any) -> Arrow:
        try:
            return arrow.get(value)
        except (TypeError, arrow.parser.ParserError) as e:
            raise ValueError(str(e))

    def __copy__(self):
        return DateTime(self._value)

    def __repr__(self):
        return self._value.format("YYYY-MM-DDTHH:mm:ss.SSSSSSZZ")

    def __eq__(self, other):
        return self._value == other._value

    def __ne__(self, other):
        return self._value != other._value

    def __ge__(self, other):
        return self._value >= other._value

    def __le__(self, other):
        return self._value <= other._value

    def __gt__(self, other):
        return self._value > other._value

    def __lt__(self, other):
        return self._value < other._value

    def _format(self, date_format) -> str:
        return self._value.to(self._timezone).format(date_format)


class Date(DateTime):
    def __init__(self, datetime: DateTimeIn):
        super().__init__(datetime)
        self._value = arrow.get(self._value.date())

    def __copy__(self):
        return Date(self._value)

    def __repr__(self):
        return self._value.format("YYYY-MM-DD")  # pragma: no cover



from injector import inject

from griff.appli.event.event import Event
from griff.appli.event.event_dispatcher import EventDispatcher, AppEventDispatcher
from griff.appli.event.event_handler import EventHandler, AppEventHandler
from griff.appli.message.message_bus import MessageBus


class EventBus(MessageBus[Event, None, EventHandler]):
    @inject
    def __init__(self, dispatcher: EventDispatcher) -> None:
        super().__init__(dispatcher)


class AppEventBus(MessageBus[Event, None, AppEventHandler]):
    @inject
    def __init__(self, dispatcher: AppEventDispatcher) -> None:
        super().__init__(dispatcher)



from pydantic import Field
from pydantic_settings import BaseSettings


class DbPoolSettings(BaseSettings):
    min_size: int = 10
    max_size: int = 50


class DbSettings(BaseSettings):
    name: str
    host: str
    port: int = 5432
    user: str
    password: str
    pool: DbPoolSettings = Field(default_factory=DbPoolSettings)

    @property
    def dsn(self) -> str:
        return (
            f"postgresql://{self.user}:{self.password}"
            f"@{self.host}:{self.port}/{self.name}"
        )

    @property
    def driver(self) -> str:
        return "asyncpg"

    @property
    def db_test_name(self) -> str:  # pragma: no cover
        return f"test_{self.name}"



import abc
from typing import TypeVar, Type

from griff.runtime.factories.abstract_runtime_factory import Injectables
from griff.runtime.runtime import Runtime
from griff.services.uniqid.uniqid_service import UniqIdService
from griff.test_utils.mixins.async_test_mixin import AsyncTestMixin
from griff.test_utils.pytest_runtime_factory import PytestRuntimeFactory

K = TypeVar("K")


class RuntimeTestMixin(AsyncTestMixin, abc.ABC):
    runtime: Runtime
    uniqid_service: UniqIdService
    _pytest_runtime_factory: PytestRuntimeFactory  # must be set in conftest.py

    @classmethod
    def setup_class(cls):
        cls.runtime = (
            cls.runtime_factory().with_injectables(cls.with_injectables()).build()
        )
        cls.uniqid_service = cls.get_injected(UniqIdService)
        cls.runtime.initialize()
        if hasattr(super(), "setup_class"):
            super().setup_class()

    @classmethod
    def teardown_class(cls):
        cls.runtime.clean()
        if hasattr(super(), "teardown_class"):
            super().teardown_class()

    def setup_method(self):
        self.runtime.start()
        if hasattr(super(), "setup_method"):
            super().setup_method()

    def teardown_method(self):
        if hasattr(super(), "teardown_method"):
            super().teardown_method()
        self.runtime.stop()

    async def async_setup(self):
        await self.runtime.async_start()

    async def async_teardown(self):
        await self.runtime.async_stop()

    @classmethod
    @abc.abstractmethod
    def runtime_factory(cls) -> PytestRuntimeFactory:
        return cls._runtime_factory()

    @classmethod
    @abc.abstractmethod
    def with_injectables(cls) -> Injectables | None:
        pass

    @classmethod
    def get_injected(cls, klass: Type[K]) -> K:
        return cls.runtime.get_injector().get(klass)

    @classmethod
    def get_settings(cls):
        return cls.runtime.get_settings()

    @classmethod
    def _runtime_factory(cls) -> PytestRuntimeFactory:
        if not hasattr(cls, "_pytest_runtime_factory"):
            raise AttributeError(
                "cls._pytest_runtime_factory must be set in conftest.py"
            )
        return cls._pytest_runtime_factory



from typing import Type, Any

from injector import Binder

from griff.runtime.components.abstract_runtime_component import (
    RuntimeComponent,
    InjectBindable,
)

Injectable = Any | Type[Any]


class InjectRuntimeComponent(InjectBindable, RuntimeComponent):
    def __init__(self, klass: Any, to: Injectable):
        self.klass = klass
        self.to = to

    def configure(self, binder: Binder) -> None:
        binder.bind(self.klass, to=self.to)



from abc import ABC, abstractmethod

from injector import inject

from griff.infra.persistence.persistence import (
    Persistence,
    QueryRowResult,
    QueryResult,
    QueryRowResults,
)
from griff.infra.persistence.serialized_persistence import SerializedPersistence
from griff.services.json.json_service import JsonService
from griff.services.query_runner.query_runner_service import QueryRunnerService


class DbPersistence(Persistence, ABC):
    @inject
    def __init__(self, query_runner_service: QueryRunnerService):
        super().__init__()
        self._query_runner_service = query_runner_service
        self._query_runner_service.set_sql_queries(
            self._get_relative_sql_queries_path()
        )

    async def _insert(self, data: dict) -> None:
        await self._run_query(query_name="insert", **data)

    async def _update(self, data: dict) -> None:
        await self._run_query(query_name="update", **data)

    async def _delete(self, persistence_id: str) -> None:
        await self._run_query(query_name="delete", entity_id=persistence_id)

    async def _get_by_id(self, persistence_id: str) -> QueryRowResult:
        return await self._run_query(query_name="get_by_id", entity_id=persistence_id)  # type: ignore # noqa: E501

    async def _list_all(self) -> QueryRowResults:
        return await self._run_query(query_name="list_all")  # type: ignore

    async def _run_query(self, query_name, **query_params) -> QueryResult:
        return await self._query_runner_service.run_query(query_name, **query_params)

    @abstractmethod
    def _get_relative_sql_queries_path(self) -> str:  # pragma: no cover
        pass


class SerializedDbPersistence(
    SerializedPersistence, DbPersistence, ABC
):  # pragma: no cover
    @inject
    def __init__(
        self, query_runner_service: QueryRunnerService, json_service: JsonService
    ):
        SerializedPersistence.__init__(self, json_service)
        DbPersistence.__init__(self, query_runner_service)



from abc import ABC

from pydantic import BaseModel, ConfigDict


class Message(BaseModel, ABC):
    model_config = ConfigDict(frozen=True)

    @classmethod
    def classname(cls) -> str:
        return str(cls)

    @classmethod
    def short_classname(cls) -> str:
        return cls.__name__



from abc import ABC, abstractmethod

from griff.services.hasher.hasher_models import UnHashedStr, HashedStr


class HasherProvider(ABC):
    @abstractmethod
    def hash(self, to_hash: UnHashedStr) -> HashedStr:  # pragma: no cover
        pass

    @abstractmethod
    def verify(
        self, candidate: UnHashedStr, reference: HashedStr
    ) -> bool:  # pragma: no cover
        pass



import csv

from injector import inject, singleton

from griff.services.abstract_service import AbstractService
from griff.services.csv.csv_models import CsvOptions
from griff.services.path.path_service import PathService


@singleton
class CsvService(AbstractService):
    @inject
    def __init__(self, path_service: PathService):
        self._path_service = path_service

    def load(self, csv_filename, options: CsvOptions | None = None):
        csv_path = self._path_service.check_exists(csv_filename)
        options = options or CsvOptions()
        with csv_path.open() as df:
            reader = csv.reader(df, delimiter=options.field_delimiter)
            headers = [c for c in next(reader)]
            return [
                {headers[i]: value for i, value in enumerate(row)}
                for row in reader
                if not options.ignore_empty_rows or any(row)
            ]



import inspect
import random
from abc import ABC
from decimal import Decimal
from typing import Self, List

from griff.domain.common_types import Entity
from griff.infra.persistence.persistence import Persistence
from griff.infra.repository.repository import Repository
from griff.services.date.fake_date_service import FakeDateService
from griff.services.service_locator.service_locator import ServiceLocator
from griff.services.uniqid.uniqid_service import UniqIdService


class DataTestFactory(ABC):
    def __init__(self, start_id=1):
        self.uniqid_service = ServiceLocator.get(UniqIdService)
        self.date_service = FakeDateService()
        self._sequence = {}
        random.seed(42)
        self._random_state = random.getstate()
        self.reset(start_id)
        self._created = {}

    def reset(self, start_id=1):
        self.uniqid_service.reset(start_id)

    async def persist(
        self, repository: Repository, data: list[Entity] | Entity
    ) -> Self:
        if isinstance(data, list) is False:
            await repository.save(data)
            return self

        for a in data:
            await repository.save(a)
        return self

    async def persist_by_persistence(
        self, persistence: Persistence, data: List[Entity] | Entity
    ) -> Self:
        initial_data = data if isinstance(data, list) else [data]
        prepared_data = [d.model_dump() for d in initial_data]
        persistence.reset(initial_data=prepared_data)
        return self

    def _random_decimal(self, min, max) -> Decimal:
        random.setstate(self._random_state)
        min = float(min) * 100
        max = float(max) * 100
        val = Decimal(random.uniform(min, max)) / 100
        self._random_state = random.getstate()
        return val

    def _random_int(self, min, max) -> int:
        random.setstate(self._random_state)
        val = random.randrange(min, max)
        self._random_state = random.getstate()
        return val

    def _next_sequence(self):
        current_frame = inspect.currentframe()
        calling_frame = inspect.getouterframes(current_frame, 2)
        caller = calling_frame[1][3]
        if caller not in self._sequence:
            self._sequence[caller] = 0
        self._sequence[caller] += 1
        # self.uniqid_service.reset(self._sequence[caller])
        return self._sequence[caller]

    def _add_created(self, name, created):
        if name not in self._created:
            self._created[name] = []
        self._created[name].append(created)
        return created

    def list_created(self, name):
        return self._created[name] if name in self._created else []



from injector import inject, singleton

from griff.infra.cli.cli_router import CliRouter
from griff.infra.cli.controllers.db_cli_controller import DbCliController
from griff.infra.cli.controllers.db_tpl.db_tpl_cli_controller import (
    DbTplCliController,
)
from griff.infra.cli.controllers.i18n_cli_controller import I18nCliController


@singleton
class CommonCliRouter(CliRouter):
    @inject
    def __init__(
        self,
        i18n: I18nCliController,
        db_tpl_ctrl: DbTplCliController,
        db_contrl: DbCliController,
    ):
        super().__init__()
        self._app.add_typer(i18n.get_app(), name=i18n.get_command_name())
        self._app.add_typer(db_tpl_ctrl.get_app(), name=db_tpl_ctrl.get_command_name())
        self._app.add_typer(db_contrl.get_app(), name=db_contrl.get_command_name())

    def get_command_group_name(self) -> str:
        return "common"



from injector import Injector

from griff.runtime.components.db_test_in_transaction_component import (
    DbTestInTransactionRuntimeComponent,
)
from griff.settings.griff_settings import GriffSettings
from griff.utils.db_tpl_utils import DbTplUtils


class DbTestContextInTransactionRuntimeComponent(DbTestInTransactionRuntimeComponent):
    def __init__(self, context: str, settings: GriffSettings):
        self._context = context
        self._settings = settings
        self._db_tpl = DbTplUtils(context=context, settings=settings)

    async def async_start(self, injector: Injector):
        await self._db_tpl.load()
        await super().async_start(injector)

    async def async_stop(self, injector: Injector):
        await super().async_stop(injector)
        await self._db_tpl.clean()



from loguru import logger

from griff.appli.command.command import Command
from griff.appli.command.command_handler import CommandResponse
from griff.appli.command.command_middleware import CommandMiddleware
from griff.appli.message.message_middleware import MessageContext


class CommandLoggerMiddleware(CommandMiddleware):
    async def dispatch(
        self, message: Command, context: MessageContext | None = None
    ) -> CommandResponse:
        logger.info(f"dispatch command: {message.short_classname()}")
        logger.debug(message.model_dump())
        response = await self._next_dispatch(message, context)
        logger.debug(response.model_dump())
        return response



from abc import ABC, abstractmethod
from typing import TypeVar, Generic

from griff.appli.message.message_handler import (
    MessageHandler,
    MessageErrorResponse,
    MessageSuccessResponse,
)
from griff.appli.query.query import Query
from griff.infra.registry.meta_registry import MetaQueryHandlerRegistry


class QuerySuccessResponse(MessageSuccessResponse, ABC):
    ...


class QueryErrorResponse(MessageErrorResponse, ABC):
    ...


QueryResponse = QuerySuccessResponse | QueryErrorResponse

QM = TypeVar("QM", bound=Query)
QR = TypeVar("QR", bound=QueryResponse)


class QueryHandler(
    Generic[QM, QR], MessageHandler[QM, QR], ABC, metaclass=MetaQueryHandlerRegistry
):
    @abstractmethod
    async def handle(self, query: QM) -> QR:
        ...



class EndpointDecoratorProcessor:
    """
    Class meant to interpret endpoint decorator.
    Such a decorator should contain :

    @register_endpoint(route="/me", method="GET",
    permission="LEAGUE_ADMIN",
    success_code=status.HTTP_201_CREATED)

    route: a str that shall start by '/'
    method: a string in [GET, POST, PATCH, PUT, OPTIONS]
    http_code: a starlette HTTP code
    permission: a string in PUBLIC / SIMPLE_USER / LEAGUE_ADMIN /SITE_ADMIN
    """

    def __init__(self):
        self._route = None
        self._method = None
        self._http_code = None

    @staticmethod
    def _sanitize(to_be_sanitized: str):
        return to_be_sanitized.strip("'").strip('"').strip(" ")

    def _check_route_is_correct(self, a_route_definition: str) -> bool:
        splited = a_route_definition.split("=")
        if self._sanitize(splited[1])[0] != "/":
            raise ValueError("route définition shall start by /")
        return True

    def _check_method_is_correct(self, a_method_definition: str):
        splited = a_method_definition.split("=")
        method = self._sanitize(splited[1])
        if method not in ["GET", "POST", "OPTIONS", "PATCH", "PUT", "DELETE"]:
            raise ValueError(f"Http method {method} unknown")
        return True

    def _check_http_code_is_correct(self, a_http_code_def: str) -> int:
        splited = a_http_code_def.split("=")
        http_code = self._sanitize(splited[1])
        # do not remove line below !!
        from starlette import status  # noqa: F401

        return eval(http_code)

    def _check_decoration_string(self, a_decoration_str: str):
        if "route" not in a_decoration_str:
            raise ValueError("route définition not found in decoration")

    def _process_route_param(self, a_str_to_process: str):
        if "route" in a_str_to_process and self._check_route_is_correct(
            a_route_definition=a_str_to_process
        ):
            splited = a_str_to_process.split("=")
            self._route = self._sanitize(splited[1])

    def _process_http_method_param(self, a_str_to_process: str):
        if "method" in a_str_to_process and self._check_method_is_correct(
            a_method_definition=a_str_to_process
        ):
            splited = a_str_to_process.split("=")
            self._method = self._sanitize(splited[1])

    def _process_http_code_param(self, a_str_to_process: str):
        if "success_code" in a_str_to_process:
            self._http_code = self._check_http_code_is_correct(
                a_http_code_def=a_str_to_process
            )

    def load_decorators(self, a_decoration_string_list: list[str]):
        for decorator in a_decoration_string_list:
            if decorator[:18] == "@register_endpoint":
                self._check_decoration_string(decorator)
                keyval_list = decorator[
                    decorator.index("(") + 1 : decorator.index(")")  # noqa: E203
                ].split(",")
                for decoration_param in keyval_list:
                    if len(decoration_param) <= 1:
                        continue
                    if "=" not in decoration_param:
                        raise ValueError("decoration badly formatted, '=' not found")
                    self._process_route_param(a_str_to_process=decoration_param)
                    self._process_http_code_param(a_str_to_process=decoration_param)
                    self._process_http_method_param(a_str_to_process=decoration_param)

    def get_decorator_params(self):
        return {
            "route": self._route,
            "method": self._method,
            "http_code": self._http_code,
        }

    @property
    def route(self):
        return self._route

    @property
    def method(self):
        return self._method

    @property
    def code(self):
        return self._http_code



from abc import ABC, abstractmethod
from typing import TypeVar, Generic, Dict, Any

from pydantic import BaseModel, Field

from griff.appli.message.message_handler import (
    Message,
    MessageResponse,
)

M = TypeVar("M", bound=Message)
MR = TypeVar("MR", bound=MessageResponse | None)


class MessageContext(BaseModel):
    context: Dict[str, Any] = Field(default_factory=dict)

    def get(self, key):
        if key not in self.context.keys():
            raise ValueError(f"{key} not found in command context")
        return self.context[key]

    def add(self, key, value):
        if key in self.context.keys():
            raise ValueError(
                f' "{key}" already set in command context, '
                f'with "{self.context[key]}" value'
            )
        self.context[key] = value

    def has(self, key) -> bool:
        return key in self.context.keys()


class MessageMiddleware(Generic[M, MR], ABC):
    def __init__(self) -> None:
        self._next: MessageMiddleware[M, MR] | None = None

    def set_next(
        self, middleware: "MessageMiddleware[M, MR]"
    ) -> "MessageMiddleware[M, MR]":
        self._next = middleware
        return middleware

    @abstractmethod
    async def dispatch(
        self, message: M, context: MessageContext | None = None
    ) -> MR:  # pragma: no cover
        ...

    async def _next_dispatch(
        self, message: M, context: MessageContext | None = None
    ) -> MR:
        if self._next is not None:
            return await self._next.dispatch(message, context)
        if self._next is None:  # pragma: no cover
            raise RuntimeError(f"No middleware found after {self.__class__.__name__}")



from griff.test_utils.testcases.testcase import TestCase


class VoTestCase(TestCase):
    @staticmethod
    def assert_comparing_failed(vo, vo2, vo_same):
        assert (vo == vo2) is False, "== failed"
        assert (vo != vo_same) is False, "!= failed"
        assert (vo > vo2) is False, "> failed"
        assert (vo2 < vo) is False, "< failed"
        assert (vo >= vo2) is False, ">=  failed"
        assert (vo2 <= vo) is False, "<=  failed"

    @staticmethod
    def assert_comparing_succeed(vo, vo2, vo_same):
        assert vo == vo_same, "== failed"
        assert vo != vo2, "!= failed"
        assert vo2 > vo, "> failed"
        assert vo < vo2, "< failed"
        assert vo2 >= vo, ">=  failed"
        assert vo_same >= vo, ">=  failed"
        assert vo <= vo2, "<=  failed"
        assert vo <= vo_same, "<=  failed"

    @staticmethod
    def assert_copy(vo):
        copy = vo.copy()
        assert copy is not vo
        assert copy == vo



from typing import Type

from injector import Injector
from loguru import logger
from typer import Typer

from griff.context_entry_point import ContextEntryPoint
from griff.runtime.components.abstract_runtime_component import (
    RuntimeComponent,
    Runnable,
)


class ContextCliRuntimeComponent(Runnable, RuntimeComponent):
    def __init__(self, context_entry_point_class: Type[ContextEntryPoint]):
        self._entry_point_class = context_entry_point_class

    def initialize(self, injector: Injector):
        entry_point = injector.get(self._entry_point_class)
        context_router = entry_point.get_cli_router()
        if context_router is None:
            logger.warning("No CLI router found for entry point {self._entry_point}")
            return None
        typer = injector.get(Typer)
        typer.add_typer(
            context_router.get_app(), name=context_router.get_command_group_name()
        )

    def clean(self, injector: Injector):
        pass

    def start(self, injector: Injector):
        pass

    def stop(self, injector: Injector):
        pass



from injector import singleton, inject

from griff.services.abstract_service import AbstractService
from griff.services.hasher.hasher_models import UnHashedStr, HashedStr
from griff.services.hasher.providers.hasher_provider import HasherProvider


@singleton
class HasherService(AbstractService):
    @inject
    def __init__(self, hasher_provider: HasherProvider):
        self._hasher_provider = hasher_provider

    def hash(self, to_hash: UnHashedStr) -> HashedStr:
        return self._hasher_provider.hash(to_hash)

    def verify(self, candidate: UnHashedStr, reference: HashedStr) -> bool:
        return self._hasher_provider.verify(candidate, reference)



from injector import Injector, Binder, singleton
from typer import Typer

from griff.infra.cli.common_cli_router import CommonCliRouter
from griff.runtime.components.abstract_runtime_component import (
    RuntimeComponent,
    Runnable,
    InjectBindable,
)


class CommonCliRuntimeComponent(InjectBindable, Runnable, RuntimeComponent):
    def configure(self, binder: Binder) -> None:
        binder.bind(Typer, to=Typer, scope=singleton)

    def initialize(self, injector: Injector):
        typer = injector.get(Typer)
        cli_router = injector.get(CommonCliRouter)
        typer.add_typer(cli_router.get_app(), name=cli_router.get_command_group_name())

    def clean(self, injector: Injector):
        pass

    def start(self, injector: Injector):
        pass

    def stop(self, injector: Injector):
        pass



from typing import TypeVar, List, Type

from injector import Injector

from griff.runtime.components.abstract_runtime_component import (
    RuntimeComponent,
    Runnable,
)
from griff.services.abstract_service import AbstractService
from griff.services.service_locator.service_locator import ServiceLocator
from griff.settings.griff_settings import GriffSettings

S = TypeVar("S", bound=GriffSettings)


class ServiceLocatorRuntimeComponent(Runnable, RuntimeComponent):
    def __init__(self, services: List[Type[AbstractService]]):
        self._services = services

    def initialize(self, injector: Injector):
        ServiceLocator.reset()
        for service in self._services:
            ServiceLocator.register(service, injector.get(service))

    def start(self, injector: Injector):
        pass

    def stop(self, injector: Injector):
        pass

    def clean(self, injector: Injector):
        ServiceLocator.reset()



from griff.appli.message.message_dispatcher import MessageDispatcher
from griff.appli.query.query import Query
from griff.appli.query.query_handler import QueryResponse


class QueryDispatcher(MessageDispatcher[Query, QueryResponse]):
    ...



from injector import inject, singleton

from griff.services.db.db_service import DbService
from griff.services.db.providers.db_provider import DbProvider


@singleton
class DbTestService(DbService):
    @inject
    def __init__(self, provider: DbProvider):
        super().__init__(provider)
        self._current_connection = None  # type: ignore

    def _get_current_connection(self):
        return self._current_connection

    def _set_current_connection(self, connection):
        self._current_connection = connection
        return None

    def _reset_current_connection(self, token):  # pragma: no cover
        self._current_connection = None



import enum
from abc import ABC
from pathlib import Path
from typing import Set

from pydantic import Field, model_validator
from pydantic_settings import BaseSettings, SettingsConfigDict

from griff.services.db.db_settings import DbSettings
from griff.services.jwt.jwt_settings import JwtSettings
from griff.services.query_runner.query_runner_settings import QueryRunnerSettings


class LogLevel(str, enum.Enum):
    CRITICAL = "CRITICAL"
    ERROR = "ERROR"
    WARNING = "WARNING"
    INFO = "INFO"
    DEBUG = "DEBUG"


class LocaleSettings(BaseSettings):
    default: str
    relative_path: str = "app/locale"
    pydantic_locale_dir_name: str = "pydantic"
    fastapi_locale_dir_name: str = "fastapi"


class CORSSettings(BaseSettings):
    allow_origins: Set[str] = {"localhost"}
    allow_credentials: bool = True
    allow_methods: Set[str] = {"*"}
    allow_headers: Set[str] = {"*"}

    # @field_validator("allow_origins", "allow_methods", "allow_headers", mode="before")
    # @classmethod
    # def str_to_set(cls, v) :
    #     return v


class GriffSettings(BaseSettings, ABC):
    model_config = SettingsConfigDict(
        validate_assignment=True,
        env_nested_delimiter="__",
        env_file=".env",
        extra="ignore",
    )

    env: str = "prod"

    project_name: str
    project_dir: str
    bounded_contexts: list[str] = Field(default_factory=list)

    debug: bool = False
    log_level: LogLevel = LogLevel.INFO

    db: DbSettings
    cors: CORSSettings = Field(default_factory=CORSSettings)

    locale: LocaleSettings | None = None

    jwt: JwtSettings | None = None
    query_runner: QueryRunnerSettings | None = None

    @model_validator(mode="after")
    def set_query_runner_dir(self):
        if self.query_runner is None:
            self.query_runner = QueryRunnerSettings(
                project_dir=self.project_dir, driver=self.db.driver
            )
        if self.query_runner.project_dir is None:
            self.query_runner.project_dir = self.project_dir
        return self

    @property
    def full_locale_path(self):
        locale = self._check_locale()
        return str(Path(self.project_dir).joinpath(locale.relative_path))

    @property
    def pydantic_locale_path(self) -> str:
        locale = self._check_locale()
        return str(
            Path(self.project_dir).joinpath(
                locale.relative_path, locale.pydantic_locale_dir_name
            )
        )

    @property
    def fastapi_local_path(self) -> str:
        locale = self._check_locale()
        return str(
            Path(self.project_dir).joinpath(
                locale.relative_path, locale.fastapi_locale_dir_name
            )
        )

    def get_app_path(self, absolute: bool = True) -> Path:
        base_path = Path(self.project_dir) if absolute else Path("./")
        return base_path.joinpath("app")

    def get_app_event_handlers_path(self, absolute: bool = True) -> Path:
        return self.get_app_path(absolute).joinpath("app_event_handlers")

    def get_context_path(self, context: str, absolute: bool = True) -> Path:
        if context not in self.bounded_contexts:
            raise ValueError(f"Context {context} is not in bounded contexts")

        base_path = Path(self.project_dir) if absolute else Path("./")
        return base_path.joinpath(context)

    def get_event_handlers_path(self, context: str, absolute: bool = True) -> Path:
        return self.get_context_path(context, absolute).joinpath("event_handlers")

    def get_common_path(self, context: str, absolute: bool = True) -> Path:
        return self.get_context_path(context, absolute).joinpath("_common")

    def get_repositories_path(self, context: str, absolute: bool = True) -> Path:
        return self.get_common_path(context, absolute).joinpath("repositories")

    def get_migrations_path(self, context: str, absolute: bool = True) -> Path:
        return self.get_repositories_path(context, absolute).joinpath("migrations")

    def get_queries_path(self, context: str, domain: str, absolute: bool = True):
        return self.get_repositories_path(context, absolute).joinpath(domain, "sql")

    def get_test_utils_path(self, context: str, absolute: bool = True) -> Path:
        return self.get_common_path(context, absolute).joinpath("test_utils")

    def _check_locale(self) -> LocaleSettings:
        if self.locale is None:
            raise ValueError("Locale settings are not set")
        return self.locale



from yoyo import get_backend, read_migrations


class MigrationUtils:
    @staticmethod
    def migrate(dsn: str, migrations_path: str) -> None:  # pragma: no cover
        backend = get_backend(dsn)
        migrations = read_migrations(str(migrations_path))

        with backend.lock():
            backend.apply_migrations(backend.to_apply(migrations))
            backend.connection.commit()



from abc import ABC

from asyncpg import Connection

from griff.runtime.runtime import Runtime
from griff.services.db.providers.asyncpg_provider import AsyncPgProvider
from griff.test_utils.mixins.runtime_test_mixin import RuntimeTestMixin
from griff.test_utils.pytest_runtime_factory import PytestRuntimeFactory


class DbTestMixin(RuntimeTestMixin, ABC):
    provider: AsyncPgProvider
    runtime: Runtime

    @classmethod
    def runtime_factory(cls) -> PytestRuntimeFactory:
        return super().runtime_factory().test_with_db()

    @classmethod
    def setup_class(cls):
        if hasattr(super(), "setup_class"):
            super().setup_class()
        cls.async_to_sync(cls.create_test_db)

    @classmethod
    def teardown_class(cls):
        if hasattr(super(), "teardown_class"):
            super().teardown_class()
        cls.async_to_sync(cls.drop_test_db)

    @classmethod
    async def create_test_db(cls):
        provider = cls._get_template1_provider()
        async with provider.connection() as conn:
            try:
                await cls._create_test_db(provider, conn)
            except Exception:
                # gestion d'un test qui aurait échoué et qui n'aurait pas fait sont
                # teardown
                await cls._disconnect_all(provider, conn)
                await cls._drop_test_db(provider, conn)
                await cls._create_test_db(provider, conn)

    @classmethod
    async def drop_test_db(cls):
        provider = cls._get_template1_provider()
        async with provider.connection() as conn:
            try:
                await cls._drop_test_db(provider, conn)
            except Exception as e:
                print(f"ERROR: {e}")

    @classmethod
    async def _create_test_db(cls, provider: AsyncPgProvider, conn: Connection):
        # noinspection SqlInjection
        sql = f"CREATE DATABASE {cls.runtime.get_settings().db.name}"
        await provider.execute(conn, sql)

    @classmethod
    async def _drop_test_db(cls, provider: AsyncPgProvider, conn: Connection):
        # noinspection SqlInjection
        sql = f"DROP DATABASE {cls.runtime.get_settings().db.name}"
        await provider.execute(conn, sql)

    @classmethod
    def _get_template1_provider(cls):
        settings = cls.runtime.get_settings().db.copy(deep=True)
        settings.name = "template1"
        return AsyncPgProvider(settings)

    @classmethod
    async def _disconnect_all(cls, provider: AsyncPgProvider, conn: Connection):
        # noinspection SqlInjection
        await cls._get_nb_active_connexion(provider, conn)
        await provider.disconnect_all(conn)

    @classmethod
    async def _get_nb_active_connexion(
        cls, provider: AsyncPgProvider, conn: Connection
    ):
        row = await provider.fetch_one(
            conn, "SELECT sum(numbackends) as nb FROM pg_stat_database;"
        )
        print(f"{row['nb']} connexion active")
        return row["nb"]



from loguru import logger

from griff.appli.message.message_middleware import MessageContext
from griff.appli.query.query import Query
from griff.appli.query.query_handler import QueryResponse
from griff.appli.query.query_middleware import QueryMiddleware


class QueryLoggerMiddleware(QueryMiddleware):
    async def dispatch(
        self, message: Query, context: MessageContext | None = None
    ) -> QueryResponse:
        logger.info(f"dispatch query: {message.short_classname()}")
        logger.debug(message.model_dump())
        response = await self._next_dispatch(message)
        logger.debug(response.model_dump())
        return response



from abc import ABC, ABCMeta
from typing import Type, Any, List


class AbstractMetaRegistry(ABCMeta, type):
    except_name = "None"
    REGISTRY: dict[str, Type[Any]]

    def __new__(cls, name: str, bases: Any, attrs: dict[str, Any]) -> Type[Any]:
        new_cls = type.__new__(cls, name, bases, attrs)
        if ABC in bases:
            # no need to register Abstract class
            return new_cls

        class_name = f"{new_cls.__module__}.{new_cls.__name__}"
        cls.REGISTRY[class_name] = new_cls
        return new_cls

    @classmethod
    def list_types(cls) -> List[Type[Any]]:
        return list(cls.REGISTRY.values())


class MetaQueryHandlerRegistry(AbstractMetaRegistry):
    REGISTRY = {}


class MetaEventHandlerRegistry(AbstractMetaRegistry):
    REGISTRY = {}


class MetaAppEventHandlerRegistry(AbstractMetaRegistry):
    REGISTRY = {}


class MetaCommandHandlerRegistry(AbstractMetaRegistry):
    REGISTRY = {}


class MetaContextEntryPointRegistry(AbstractMetaRegistry):
    REGISTRY = {}


class MetaApiRouterRegistry(AbstractMetaRegistry):
    REGISTRY = {}


class MetaCliRouterRegistry(AbstractMetaRegistry):
    REGISTRY = {}



from abc import ABC, abstractmethod
from typing import Generic, TypeVar, List

from griff.appli.message.message import Message
from griff.appli.message.message_dispatcher import MessageDispatcher
from griff.appli.message.message_handler import MessageResponse, MessageHandler
from griff.appli.message.message_middleware import MessageMiddleware, MessageContext

M = TypeVar("M", bound=Message)
MR = TypeVar("MR", bound=MessageResponse)
MH = TypeVar("MH", bound=MessageHandler)
ListMiddlewares = List[MessageMiddleware]


class MessageBus(Generic[M, MR, MH], ABC):
    @abstractmethod
    def __init__(self, dispatcher: MessageDispatcher) -> None:
        self._dispatcher = dispatcher
        self._middleware: MessageMiddleware | None = None

    def initialize(self, handlers: List[MH], middlewares: ListMiddlewares) -> None:
        self._register_handlers(handlers)
        self._register_middlewares(middlewares)

    async def dispatch(self, message: M, context: MessageContext | None = None) -> MR:
        if self.is_initialized() is False:
            raise RuntimeError(f"{self.__class__.__name__} not initialized")
        return await self._middleware.dispatch(message, context)  # type: ignore

    def is_initialized(self) -> bool:
        return self._middleware is not None

    def _register_middlewares(self, middlewares: ListMiddlewares) -> None:
        # dispatcher must be the last one
        all_middlewares = middlewares + [self._dispatcher]
        middleware = all_middlewares[0]
        for next_middleware in all_middlewares[1:]:
            middleware.set_next(next_middleware)
            middleware = next_middleware
        self._middleware = all_middlewares[0]

    def _register_handlers(self, handlers: List[MH]) -> None:
        self._dispatcher.register(handlers)



from typing import TypeVar

from injector import Binder, Injector

from griff.runtime.components.abstract_runtime_component import (
    RuntimeComponent,
    InjectBindable,
    AsyncRunnable,
)
from griff.services.db.db_service import DbService
from griff.services.db.db_settings import DbSettings
from griff.services.db.providers.asyncpg_provider import AsyncPgProvider
from griff.services.db.providers.db_provider import DbProvider
from griff.settings.griff_settings import GriffSettings

S = TypeVar("S", bound=GriffSettings)


class DbRuntimeComponent(AsyncRunnable, InjectBindable, RuntimeComponent):
    def __init__(self, settings: S):
        self._settings = settings

    def configure(self, binder: Binder) -> None:
        binder.bind(DbSettings, self._settings.db)
        binder.bind(DbProvider, AsyncPgProvider)  # type: ignore

    async def async_start(self, injector: Injector):
        db_service = injector.get(DbService)
        await db_service.start()

    async def async_stop(self, injector: Injector):
        db_service = injector.get(DbService)
        await db_service.stop()


class DbTestRuntimeComponent(InjectBindable, RuntimeComponent):
    def __init__(self, settings: S):
        self._settings = settings

    def configure(self, binder: Binder) -> None:
        binder.bind(DbSettings, self._settings.db)
        binder.bind(DbProvider, AsyncPgProvider)  # type: ignore
